import 'package:flutter/cupertino.dart';
import 'package:nui_auth/domain/nui_user_activity.dart';
import 'package:nui_auth/web/nui_auth_web_json.dart';
import 'package:nui_core/model/interfaces.dart';
import 'package:nui_web/nui_web.dart';
import '../domain/nui_user_activity.dart';
import '../domain/nui_user_info.dart';

abstract class _Apis{
  Future<NUIStandardHttpResponse> signup({@required NUIAuthSignUpReq request}); //Sign up
  Future<DataResult<NUIAuthLoginResp>> ssoLogin({@required NUIAuthSSOSignUpReq request}); //Login with SSO (Single Sign On)
  Future<NUIStandardHttpResponse> verifyEmail({@required NUIAuthVerifyEmailReq request}); //Verify email after registration
  Future<NUIStandardHttpResponse> resendEmailVerificationCode({@required NUIAuthVerifyEmailReq request}); //Resend email verification code
  Future<DataResult<NUIAuthLoginResp>> login({@required NUIAuthLoginReq request}); //Login
  Future<DataResult<NUIAuthLoginResp>> refreshToken({@required NUIAuthLoginReq request}); //Refresh token
  Future<NUIStandardHttpResponse> changePassword({NUIAuthChangePasswordReq request}); //Channge password
  Future<NUIStandardHttpResponse> forgotPassword({@required NUIAuthForgotPasswordReq request}); //Forgot password
  Future<NUIStandardHttpResponse> checkTokenValidity(); //Check token's validity
  Future<NUIStandardHttpResponse> logout({@required NUIAuthLogoutReq request}); //Logout
  Future<NUIStandardHttpResponse> updateUserStatusOnline(); //Update status to online
  Future<NUIStandardHttpResponse> updateUserStatusOffline(); //Update status to offline
  Future<NUIStandardHttpResponse> updateUserStatusIdle(); //Update status to idle
  Future<DataResult<NUIAuthRenewTokenResp>> renewAccessToken({@required NUIAuthRenewTokenReq request});
  Future<NUIStandardHttpResponse> logUserActivity({@required NUIUserActivity activity}); //Save user log activity
  Future<NUIStandardHttpResponse> updateUserInfo({@required NUIUserInfo info}); //Save user information
}

class NUIAuthApi implements _Apis{
  static const String MODULE = "NUIAuthApiModule";

  NUIWeb _getWeb() => NUIWeb.get(module: MODULE);

  void _processResponseMessage(NUIStandardHttpResponse response){
    if(response?.data?.containsKey("message") == true){
      response.message = response.data["message"];
    }
  }

  @override
  Future<NUIStandardHttpResponse> changePassword({NUIAuthChangePasswordReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/change-password", bodyEnt: request, entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<NUIStandardHttpResponse> checkTokenValidity() async{
    final result = await _getWeb().doPOST(url: "auth/token/auth");
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<NUIStandardHttpResponse> forgotPassword({NUIAuthForgotPasswordReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/forgot-password", bodyEnt: request, entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<NUIStandardHttpResponse> logUserActivity({NUIUserActivity activity}) async{
    final result = await _getWeb().doPOST(url: "user-activities", bodyEnt: activity, entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<NUIStandardHttpResponse> updateUserInfo({NUIUserInfo info}) async{
    final result = await _getWeb().doPATCH(url: "user-info", bodyEnt: info, entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<DataResult<NUIAuthLoginResp>> login({NUIAuthLoginReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/login", bodyEnt: request, entity: NUIAuthLoginRespEnt());
    return DataResult<NUIAuthLoginResp>(success: result.success, message: result.message, data: result.body);
  }

  @override
  Future<DataResult<NUIAuthRenewTokenResp>> renewAccessToken({NUIAuthRenewTokenReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/renewAccessToken", bodyEnt: request, entity: NUIAuthRenewTokenRespEnt());
    return DataResult<NUIAuthRenewTokenResp>(success: result.success, message: result.message, data: result.body);
  }

  @override
  Future<NUIStandardHttpResponse> logout({NUIAuthLogoutReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/logout", bodyEnt: request, entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<DataResult<NUIAuthLoginResp>> refreshToken({NUIAuthLoginReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/login", bodyEnt: request, entity: NUIAuthLoginRespEnt());
    return DataResult<NUIAuthLoginResp>(success: result.success, message: result.message, data: result.body);
  }

  @override
  Future<NUIStandardHttpResponse> resendEmailVerificationCode({NUIAuthVerifyEmailReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/signup/verify/resend", bodyEnt: request, entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<NUIStandardHttpResponse> signup({NUIAuthSignUpReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/signup", bodyEnt: request, entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<DataResult<NUIAuthLoginResp>> ssoLogin({request}) async{
    final result = await _getWeb().doPOST(url: "auth/sso-login", bodyEnt: request, entity: NUIAuthLoginRespEnt());
    return DataResult<NUIAuthLoginResp>(success: result.success, message: result.message, data: result.body);
  }

  @override
  Future<NUIStandardHttpResponse> updateUserStatusIdle() async{
    final result = await _getWeb().doPUT(url: "users/activity/idle", entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<NUIStandardHttpResponse> updateUserStatusOffline() async{
    final result = await _getWeb().doPUT(url: "users/activity/offline", entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<NUIStandardHttpResponse> updateUserStatusOnline() async{
    final result = await _getWeb().doPUT(url: "users/activity/online", entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }

  @override
  Future<NUIStandardHttpResponse> verifyEmail({NUIAuthVerifyEmailReq request}) async{
    final result = await _getWeb().doPOST(url: "auth/signup/verify", bodyEnt: request, entity: NUIStandardHttpResponseEntity());
    _processResponseMessage(result.body);
    return result.body;
  }
}